import sys
import os
from Bio import SeqIO

#grep Evidence uniprot-reviewed%3Ayes.tab | cut -f1,2,6,8,10> uniprot_evidence.tab
#awk -F'\t' '{if($2=="A" || $2=="B")print $0}' uniprot-taxcode.tab | cut -f1 > uniprot_proknames.txt
#grep -f uniprot_proknames.txt uniprot_evidence.tab > prok_evidence.tab
#awk -F'\t' '{if(length($5) && index($5,"-")==0 && index($5,";")==0) print $0}' prok_evidence.tab > prok_sel.tab
#cut -f1 prok_sel.tab > prok_sel_names.txt


def mean(l):
	sum = 0
	for x in l:
		sum += x

	return sum * 1.0 / len(l)

def create_db(info_file, tax_map, csa_file, seq_file, out_prefix):
	prok_names = []
	lines = [l.strip() for l in open(tax_map)]
	for l in lines:
		x = l.split('\t')
		if len(x) > 1:
			if x[1] == "A" or x[1] == "B":
				prok_names.append(x[0])
				
	lines = [l.strip() for l in open(info_file)]
	info_out = out_prefix + ".info.tab"
	of = open(info_out,"w")
	sel_prot = []
	sel_ec = []
	count = 0
	for l in lines[1:]:
		x = l.split('\t')
		if len(x) > 9:
			p = x[0]
			tax = x[1].split('_')[1]
			org = x[5]
			ev = x[7]
			ec = x[9]
			if tax in prok_names:
				if "Evidence" in ev:
					if ec != "" and "-" not in ec and ';' not in ec:
						s = p +'\t' + x[1] + '\t' + org + '\t' + ev + '\t' + ec +'\n'
						of.write(s)
						sel_prot.append(p)
						sel_ec.append(ec)
						count += 1

	print("Num Selected Prokaryotic Proteins before csa: " + str(count))
	clines = [l.strip() for l in open(csa_file)]
	csa_prot = []
	csa_ec = []
	csa_count = 0
	for l in clines:
		x = l.split('\t')
		plist = x[1]
		ec = x[3]
		if ':' in plist:
			y = plist.split(':')
			for p in y:
				csa_prot.append(p)
				csa_ec.append(ec)
				csa_count += 1
		else:
			csa_prot.append(plist)
			csa_ec.append(ec)
			csa_count += 1

	print("Num CSA Proteins: " + str(csa_count))

	out_seq = []
	out_ec = []
	out_lengths = []
	fasta_seq = SeqIO.parse(open(seq_file),'fasta')
	for f in fasta_seq:
		name, seq = f.id, str(f.seq)
		pn = name.split('|')[1]
		if pn in sel_prot:
			out_lengths.append(len(seq))
			out_seq.append(f)
			out_ec.append(sel_ec[sel_prot.index(pn)])
		elif pn in csa_prot:
			sp = name.split('|')[2].split('_')[1]
			ec = csa_ec[csa_prot.index(pn)]
			if sp in prok_names and '-' not in ec:
				sp_name = f.description.split("OS=")[1].split(" OX=")[0]
				s = pn +'\t' + name.split('|')[2] + '\t' + sp_name + '\t' + 'Evidence in CSA' + '\t' + ec +'\n'
				of.write(s)
				print(s)
				out_lengths.append(len(seq))
				out_seq.append(f)
				out_ec.append(ec)

	of.close()
	out_ec_lengths = out_prefix+"_ec_lengths.tab"
	ec_len = {}
	for i in xrange(len(out_ec)):
		if out_ec[i] not in ec_len.keys():
			ec_len[out_ec[i]] = [out_lengths[i]]
		else:
			ec_len[out_ec[i]].append(out_lengths[i])

	of = open(out_ec_lengths,"w")
	of.write("ec+\t+avg_length+\n")
	for e in ec_len.keys():
		avg_len = mean(ec_len[e])
		s = e + '\t' + str(avg_len) + '\n'
		of.write(s)

	of.close()

	out_fasta = out_prefix + ".fasta"
	with open(out_fasta,"w") as output_handle:
		SeqIO.write(out_seq, output_handle, "fasta")

	out_label = out_prefix + ".label"
	of = open(out_label,"w")
	for l in out_ec:
		of.write(l+'\n')

	of.close()

def main():
	if len(sys.argv) < 6:
		print("Usage: create_db.py <info_file> <tax_map> <csa_curated_db> <uni_fasta> <out_prefix>")

	else:
		info_file = sys.argv[1]
		tax_map = sys.argv[2]
		csa_file = sys.argv[3]
		seq_file = sys.argv[4]
		out_prefix = sys.argv[5]
		create_db(info_file, tax_map, csa_file, seq_file, out_prefix)


if __name__ == "__main__":
	main()
